@@ -86,7 +86,6 @@ enum {
 #define KEY_SCROLL_ON_KEYSTROKE "scroll_on_keystroke"
 #define KEY_SCROLL_ON_OUTPUT "scroll_on_output"
 #define KEY_SILENT_BELL "silent_bell"
-#define KEY_UPDATE_RECORDS "update_records"
 #define KEY_USE_CUSTOM_COMMAND "use_custom_command"
 #define KEY_USE_SYSTEM_FONT "use_system_font"
 #define KEY_USE_THEME_COLORS "use_theme_colors"
@@ -425,9 +424,6 @@ migrate_profile (TerminalSettingsList *list,
   migrate_bool (client, path, KEY_LOGIN_SHELL,
                 settings, TERMINAL_PROFILE_LOGIN_SHELL_KEY,
                 FALSE);
-  migrate_bool (client, path, KEY_UPDATE_RECORDS,
-                settings, TERMINAL_PROFILE_UPDATE_RECORDS_KEY,
-                FALSE);
   migrate_bool (client, path, KEY_USE_CUSTOM_COMMAND,
                 settings, TERMINAL_PROFILE_USE_CUSTOM_COMMAND_KEY,
                 FALSE);
